/*
 * Copyright 2019-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */

// -----------------------------------------------------------------------------
// Includes
// -----------------------------------------------------------------------------

#include <stdio.h>

#include "fsl_debug_console.h"

#include "LLBSP_impl.h"
#include "microej.h"

#include "mej_log.h"

// -----------------------------------------------------------------------------
// Macros and Defines
// -----------------------------------------------------------------------------

#define LLBSP_IS_PTR_IN_INTERNAL_RAM(p)\
	((p > &m_data_start) && (p < &m_data_end))

#define LLBSP_IS_PTR_IN_EXTERNAL_RAM(p) \
	((p > &m_ext_data_start) && (p < &m_ext_data_end))

#define LLBSP_IS_PTR_IN_RAM(p)					\
	(	0										\
		|| LLBSP_IS_PTR_IN_INTERNAL_RAM(ptr)	\
		|| LLBSP_IS_PTR_IN_EXTERNAL_RAM(ptr)	\
		)

// -----------------------------------------------------------------------------
// Static Variables
// -----------------------------------------------------------------------------

/*
 * These symbols are exported by the linker
 */

/*
 * @brief start of the internal RAM
 */
extern char m_data_start;

/*
 * @brief end of the internal RAM
 */
extern char m_data_end;

/*
 * @brief start of the external RAM
 */
extern char m_ext_data_start;

/*
 * @brief end of the external RAM
 */
extern char m_ext_data_end;

// -----------------------------------------------------------------------------
// Java Low Level APIs
// -----------------------------------------------------------------------------

// See the header file for the function documentation
uint8_t LLBSP_IMPL_isInReadOnlyMemory(void* ptr) {
	if (LLBSP_IS_PTR_IN_RAM(ptr)) {
		return MICROEJ_FALSE;
	} else { 
		return MICROEJ_TRUE;
	}
}

// See the header file for the function documentation
void LLBSP_IMPL_putchar(int32_t c) {
	PUTCHAR(c);
}

// -----------------------------------------------------------------------------
// EOF
// -----------------------------------------------------------------------------

