/*
 * Java
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be found with this software.
 */
package com.microej.path.chart;

import static com.verisilicon.vivante.vg_lite.VG_LITE_FILL_EVEN_ODD;

import com.microej.demo.watch.util.path.GradientPathGraphics;
import com.microej.demo.watch.util.path.GradientStyle;
import com.microej.vivante.utils.GradientPath;
import com.microej.vivante.utils.Path;
import com.microej.vivante.utils.PathBufferS32;
import com.verisilicon.vivante.vg_lite_gradient;
import com.verisilicon.vivante.vg_lite_matrix;
import com.verisilicon.vivante.vg_lite_path;

import ej.microui.display.GraphicsContext;

/**
 * A vector graphics that represents the chart axis.
 * <p>
 * The vector graphics is made of one rectangle of 1 pixel height, which is drawn multiple times.
 */
public class ChartAxis extends GradientPathGraphics {

	private static final int THICKNESS = 1;

	private static final float GRADIENT_SIZE = 256f;

	private static final int LINE_SPACING = 20;

	private static final int START_Y = 25;

	private static final int LINE_COUNT = 6;

	private final int translateY;

	/**
	 * Creates the axis lines.
	 *
	 * @param y
	 *            the top y-coordinate of the vector graphics.
	 * @param height
	 *            the height of the vector graphics.
	 * @param width
	 *            the width of the vector graphics.
	 * @param gradientStyle
	 *            the gradient information.
	 */
	public ChartAxis(int y, int height, int width, GradientStyle gradientStyle) {
		super(createPath(width), gradientStyle);
		this.translateY = y + height;
		initializeGradient(y, width, gradientStyle.getAngle());
	}

	private void initializeGradient(int y, int width, int angle) {
		vg_lite_gradient gradient = getGradient();
		vg_lite_matrix gradientMatrix = gradient.getMatrix();
		vg_lite_matrix.vg_lite_identity(gradientMatrix);
		vg_lite_matrix.vg_lite_translate(0, y, gradientMatrix);
		vg_lite_matrix.vg_lite_rotate(angle, gradientMatrix);
		vg_lite_matrix.vg_lite_scale(width / GRADIENT_SIZE, 1f, gradientMatrix);
	}

	private static GradientPath createPath(int width) {
		PathBufferS32 pb = new PathBufferS32();
		pb.move(0, 0);
		pb.line(0, -THICKNESS);
		pb.line(width, -THICKNESS);
		pb.line(width, 0);
		pb.line(0, 0);
		pb.close();
		vg_lite_path path = pb.toVGLitePath(0, -THICKNESS, width, THICKNESS, vg_lite_path.VG_LITE_HIGH);
		return new GradientPath(path, new vg_lite_gradient(), VG_LITE_FILL_EVEN_ODD);
	}

	@Override
	public void render(GraphicsContext g) {
		int lineY = START_Y;
		for (int i = 0; i < LINE_COUNT; i++) {
			drawLine(g, lineY);
			lineY += LINE_SPACING;
		}
	}

	private void drawLine(GraphicsContext g, int lineY) {
		Path path = getPath();
		assert path != null;

		vg_lite_matrix matrix = getMatrix();
		vg_lite_matrix.vg_lite_identity(matrix);
		vg_lite_matrix.vg_lite_translate(0, (float) this.translateY - lineY, matrix);
		getImage().draw(g, getMatrix());
	}
}
