/*
 * Java
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be found with this software.
 */
package com.microej.path.chart;

import static com.verisilicon.vivante.vg_lite_path.VG_LITE_HIGH;

import com.microej.demo.watch.util.path.GradientPathGraphics;
import com.microej.demo.watch.util.path.GradientStyle;
import com.microej.vivante.utils.GradientPath;
import com.microej.vivante.utils.PathBufferFP32;
import com.verisilicon.vivante.vg_lite;
import com.verisilicon.vivante.vg_lite_gradient;
import com.verisilicon.vivante.vg_lite_matrix;
import com.verisilicon.vivante.vg_lite_path;

/**
 * A vector graphics that represents the chart's time indicators.
 */
public class ChartTimezoneA extends GradientPathGraphics {

	private static final float GRADIENT_SIZE = 256f;
	private static final float SVG_BOUND_WIDTH = 400f;
	private static final float SVG_BOUND_HEIGHT = 400f;

	private final float translateY;
	private final int width;

	private float scale;

	/**
	 * Constructs the path of the vector graphics, given its y-position, its height and its gradient information.
	 *
	 * @param y
	 *            the top y-coordinate of the vector graphics.
	 * @param width
	 *            the width of the vector graphics.
	 * @param height
	 *            the height of the vector graphics.
	 * @param gradientStyle
	 *            the gradient information.
	 */
	public ChartTimezoneA(int y, int width, int height, GradientStyle gradientStyle) {
		super(createPath(), gradientStyle);
		this.translateY = y + height;
		this.scale = 1f;
		this.width = width;
		initializeGradient(y, height, gradientStyle.getAngle());
	}

	private static GradientPath createPath() {

		PathBufferFP32 pathBuffer = new PathBufferFP32();
		pathBuffer.setOffset(0F, -140F);
		pathBuffer.move(398.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(378.0f, 0.0f);
		pathBuffer.lineRel(-5.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(353.0f, 0.0f);
		pathBuffer.lineRel(-5.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(328.0f, 0.0f);
		pathBuffer.lineRel(-5.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(303.0f, 0.0f);
		pathBuffer.lineRel(-5.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(278.0f, 0.0f);
		pathBuffer.lineRel(-5.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(253.0f, 0.0f);
		pathBuffer.lineRel(-5.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(228.0f, 0.0f);
		pathBuffer.lineRel(-5.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(203.0f, 0.0f);
		pathBuffer.lineRel(-5.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(178.0f, 0.0f);
		pathBuffer.lineRel(-5.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(153.0f, 0.0f);
		pathBuffer.lineRel(-5.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(128.0f, 0.0f);
		pathBuffer.lineRel(-5.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(103.0f, 0.0f);
		pathBuffer.lineRel(-5.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(78.0f, 0.0f);
		pathBuffer.lineRel(-5.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(53.0f, 0.0f);
		pathBuffer.lineRel(-5.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(28.0f, 0.0f);
		pathBuffer.lineRel(-5.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 127.19999694824219f);
		pathBuffer.cubicRel(0.0f, 5.5f, -4.5f, 10.0f, -10.0f, 10.0f);
		pathBuffer.cubicRel(-5.5f, 0.0f, -10.0f, -4.5f, -10.0f, -10.0f);
		pathBuffer.lineRel(0.0f, 0.0f);
		pathBuffer.line(3.0f, 0.0f);
		pathBuffer.line(0.0f, 0.0f);
		pathBuffer.lineRel(0.0f, 140.0f);
		pathBuffer.lineRel(400.0f, 0.0f);
		pathBuffer.line(400.0f, 0.0f);
		pathBuffer.line(398.0f, 0.0f);
		vg_lite_path path = pathBuffer.toVGLitePath(-SVG_BOUND_WIDTH, -SVG_BOUND_HEIGHT, SVG_BOUND_WIDTH,
				SVG_BOUND_HEIGHT, VG_LITE_HIGH);

		return new GradientPath(path, new vg_lite_gradient(), vg_lite.VG_LITE_FILL_EVEN_ODD);
	}

	@Override
	protected void updateMatrices() {
		vg_lite_matrix matrix = getMatrix();
		vg_lite_matrix.vg_lite_identity(matrix);
		float scale = this.scale;
		vg_lite_matrix.vg_lite_translate(SVG_BOUND_WIDTH / 2 * (1f - scale), this.translateY, matrix);
		scale *= this.width;
		scale /= SVG_BOUND_WIDTH;
		vg_lite_matrix.vg_lite_scale(scale, scale, matrix);
	}

	private void initializeGradient(int y, int height, int angle) {
		vg_lite_gradient gradient = getGradient();
		vg_lite_matrix gradientMatrix = gradient.getMatrix();
		vg_lite_matrix.vg_lite_identity(gradientMatrix);
		vg_lite_matrix.vg_lite_translate(0, y, gradientMatrix);
		vg_lite_matrix.vg_lite_rotate(angle, gradientMatrix);
		vg_lite_matrix.vg_lite_scale(height / GRADIENT_SIZE, 1f, gradientMatrix);
	}

	/**
	 * Sets the scale of the vector graphics.
	 *
	 * @param scale
	 *            the scale factor.
	 */
	public void setScale(float scale) {
		this.scale = scale;
		updateMatrices();
	}

}
