/*
 * Java
 *
 * Copyright 2016-2017 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.binary;

import java.io.IOException;

/**
 * Writes GIML binary data into a stream.<p>
 * Usage of this class consists in:
 * <ol>
 * 	<li>creating a new instance</li>
 * 	<li>writing the data with several calls to write*() or {@link #markPosition(String)} methods</li>
 * 	<li>flush the data with the {@link #complete()} method</li>
 * </ol>
 */
public interface GIMLWriter {

	/**
	 * Writes a 32-bits signed integer.
	 * @param value the value to write.
	 */
	public void writeVarSInt(int value);

	/**
	 * Writes a 32-bits unsigned integer.
	 * @param value the value to write.
	 */
	public void writeVarUInt(int value);

	/**
	 * Writes a 64-bits signed integer.
	 * @param value the value to write.
	 */
	public void writeVarLong(long value);

	/**
	 * Writes a boolean.
	 * @param value the value to write.
	 */
	public void writeBoolean(boolean value);

	/**
	 * Writes a 32-bits float.
	 * @param value the value to write.
	 */
	public void writeFloat(float value);

	/**
	 * Writes a 64-bits float.
	 * @param value the value to write.
	 */
	public void writeDouble(double value);

	/**
	 * Writes a String that is not internationalized.
	 * @param s the String to write
	 */
	public void writeNonNLSString(String s);

	/**
	 * Writes a reference to an internationalized String.
	 * @param nlsId the ID of the String in the NLS system.
	 */
	public void writeNLSStringId(int nlsId);

	/**
	 * Add an identifier at the current position in the GIML stream.
	 * This identifier may be used at runtime to instantiate an object.
	 * @param identifier the identifier to define.
	 * @throws IllegalArgumentException if a position has already been marked with the given identifier.
	 */
	public void markPosition(int identifier);

	/**
	 * Generates all the content of the file. No write action is allowed after
	 * calling this method.
	 * @param binaryId the GIML binary identifier.
	 * @throws IOException if an I/O error occurs.
	 */
	public void complete(long binaryId) throws IOException;
}
