/*
 * Java
 *
 * Copyright 2016-2017 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.binary.test;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.IOException;

import org.junit.Before;
import org.junit.Test;

import com.is2t.giml.binary.GIMLWriterCompacted;
import com.is2t.nls.StringEncoder;

/**
 *
 */
public class GIMLWriterCompactedTest {

	private static final int HEADER_SIZE = 12;

	public GIMLWriterCompacted writer;
	private ByteArrayOutputStream out;

	@Before
	public void initWriter(){
		if(this.out != null){
			try {
				this.out.close();
			} catch (IOException e) {
			}
		}
		this.out = new ByteArrayOutputStream();
		initWriter(new GIMLWriterCompacted(out));
	}

	public void initWriter(GIMLWriterCompacted writer){
		this.writer = writer;
	}

	/**
	 * Test method for {@link com.is2t.giml.binary.GIMLWriterCompacted#writeVarUInt(int)}.
	 */
	@Test
	public void testWriteVarUInt() {

		writeVarUInt();

		byte[] expectedData = new byte[]{
				// Test 1
				0,
				27<<1,
				// Test 2
				(byte)0b11111110,
				(byte)0b00000011, (byte)0b00000000,
				(byte)0b00000011, (byte)0b00000000+(26<<1),
				// Test 3
				(byte)0b11111111, (byte)0b11111110,
				(byte)0b00000011, (byte)0b00000001, (byte)0b00000000,
				(byte)0b00000011, (byte)0b00000001, (byte)0b00000000+(26<<1),
				// Test 4
				(byte)0b11111111, (byte)0b11111111, (byte)0b11111110,
				(byte)0b00000011, (byte)0b00000001, (byte)0b00000001, (byte)0b00000000,
				(byte)0b00000011, (byte)0b00000001, (byte)0b00000001, (byte)0b00000000+(26<<1),
				// Test 5
				(byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111110,
				(byte)0b00000011, (byte)0b00000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000000,
				(byte)0b00000011, (byte)0b00000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000000+(26<<1),
				// Test 6
				(byte)0b00011111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111110,
				// Test 7
				// 0b0101010
				(byte)0b01010100,
				// 0b10010010110110
				(byte)0b10010011, (byte)0b01101100,
				// 0b1010101_0011001_1100110
				(byte)0b10101011, (byte)0b00110011, (byte)0b11001100,
				// 0b1110_0010101_0101100_0111101_1010010
				(byte)0b00011101, (byte)0b00101011, (byte)0b01011001, (byte)0b01111011, (byte)0b010100100,
				// 0b1010_0010101_0101100_0111101_1010010
				(byte)0b00010101, (byte)0b00101011, (byte)0b01011001, (byte)0b01111011, (byte)0b010100100,
				// 0b0110_0010101_0101100_0111101_1010010
				(byte)0b00001101, (byte)0b00101011, (byte)0b01011001, (byte)0b01111011, (byte)0b010100100,
		};

		testDataContent("writeVarUInt", expectedData);
	}

	public void writeVarUInt(){
		// Test 1
		int value = 0;
		writer.writeVarUInt(value);//encoded on 1 byte
		writer.writeVarUInt(value+27);//encoded on 1 byte

		// Test 2
		value = 0b1111111;
		writer.writeVarUInt(value);//encoded on 1 byte
		writer.writeVarUInt(value+1);//encoded on 2 bytes
		writer.writeVarUInt(value+27);//encoded on 2 bytes

		// Test 3
		value = 0b111111_11111111;
		writer.writeVarUInt(value);//encoded on 2 bytes
		writer.writeVarUInt(value+1);//encoded on 3 bytes
		writer.writeVarUInt(value+27);//encoded on 3 bytes

		// Test 4
		value = 0b11111_11111111_11111111;
		writer.writeVarUInt(value);//encoded on 3 bytes
		writer.writeVarUInt(value+1);//encoded on 4 bytes
		writer.writeVarUInt(value+27);//encoded on 4 bytes

		// Test 5
		value = 0b1111_11111111_11111111_11111111;
		writer.writeVarUInt(value);//encoded on 4 bytes
		writer.writeVarUInt(value+1);//encoded on 5 bytes
		writer.writeVarUInt(value+27);//encoded on 5 bytes

		// Test 6
		value = 0b11111111_11111111_11111111_11111111;
		writer.writeVarUInt(value);//encoded on 5 bytes

		// Test 7
		writer.writeVarUInt(0b0101010);
		writer.writeVarUInt(0b10010010110110);
		writer.writeVarUInt(0b101010100110011100110);
		writer.writeVarUInt(0b11100010101010110001111011010010);
		writer.writeVarUInt(0b10100010101010110001111011010010);
		writer.writeVarUInt(0b01100010101010110001111011010010);
	}

	/**
	 * Test method for {@link com.is2t.giml.binary.GIMLWriterCompacted#writeVarSInt(int)}.
	 */
	@Test
	public void testWriteVarSInt() {
		writeVarSInt();

		byte[] expectedData = new byte[]{
				// Test 1
				0,
				27<<1,
				// Test 2
				(byte)0b01111110,
				(byte)0b10000000,
				(byte)0b00000001, (byte)0b10000000,
				(byte)0b11111111, (byte)0b01111110,
				// Test 3
				(byte)0b01111111, (byte)0b11111110,
				(byte)0b10000001, (byte)0b00000000,
				(byte)0b00000001, (byte)0b10000001, (byte)0b00000000,
				(byte)0b11111111, (byte)0b01111111, (byte)0b11111110,
				// Test 4
				(byte)0b01111111, (byte)0b11111111, (byte)0b11111110,
				(byte)0b10000001, (byte)0b00000001, (byte)0b00000000,
				(byte)0b00000001, (byte)0b10000001, (byte)0b00000001, (byte)0b00000000,
				(byte)0b11111111, (byte)0b01111111, (byte)0b11111111, (byte)0b11111110,
				// Test 5
				(byte)0b01111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111110,
				(byte)0b10000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000000,
				(byte)0b00000001, (byte)0b10000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000000,
				(byte)0b11111111, (byte)0b01111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111110,
				// Test 6
				//Integer.MIN_VALUE
				(byte)0b11110001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000000,
				//Integer.MAX_VALUE
				(byte)0b00001111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111110,
				//-1
				(byte)0b11111110,
				// 0b1111_1111111_1111111_1111111_1111110
				(byte)0b11111100,
				// 0b1111_1111111_1111111_1111110_1111111
				(byte)0b11111101, (byte)0b11111110,
				// 0b1111_1111111_1111110_1111111_1111111
				(byte)0b11111101, (byte)0b11111111, (byte)0b11111110,
				// 0b1111_1111110_1111111_1111111_1111111
				(byte)0b11111101, (byte)0b11111111, (byte)0b11111111, (byte)0b11111110,
				// 0b1110_1111111_1111111_1111111_1111111
				(byte)0b11111101, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111110,
				// 0b1010_1111111_1111111_1111111_1111111
				(byte)0b11110101, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111110,
				// 0b0110_1111111_1111111_1111111_1111111
				(byte)0b00001101, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111110,
		};

		testDataContent("writeVarSInt", expectedData);
	}

	public void writeVarSInt() {
		// Test 1
		int value = 0;
		int negValue;
		writer.writeVarSInt(value);//encoded on 1 byte
		writer.writeVarSInt(value+27);//encoded on 1 byte

		// Test 2
		value = 0b111111;
		negValue = -(value+1);
		writer.writeVarSInt(value);//encoded on 1 byte
		writer.writeVarSInt(negValue);//encoded on 1 byte
		writer.writeVarSInt(value+1);//encoded on 2 bytes
		writer.writeVarSInt(negValue-1);//encoded on 2 bytes

		// Test 3
		value = 0b11111_11111111;
		negValue = -(value+1);
		writer.writeVarSInt(value);//encoded on 2 bytes
		writer.writeVarSInt(negValue);//encoded on 2 bytes
		writer.writeVarSInt(value+1);//encoded on 3 bytes
		writer.writeVarSInt(negValue-1);//encoded on 3 bytes

		// Test 4
		value = 0b1111_11111111_11111111;
		negValue = -(value+1);
		writer.writeVarSInt(value);//encoded on 3 bytes
		writer.writeVarSInt(negValue);//encoded on 3 bytes
		writer.writeVarSInt(value+1);//encoded on 4 bytes
		writer.writeVarSInt(negValue-1);//encoded on 4 bytes

		// Test 5
		value = 0b111_11111111_11111111_11111111;
		negValue = -(value+1);
		writer.writeVarSInt(value);//encoded on 4 bytes
		writer.writeVarSInt(negValue);//encoded on 4 bytes
		writer.writeVarSInt(value+1);//encoded on 5 bytes
		writer.writeVarSInt(negValue-1);//encoded on 5 bytes

		// Test 6
		writer.writeVarSInt(Integer.MIN_VALUE);
		writer.writeVarSInt(Integer.MAX_VALUE);
		writer.writeVarSInt(-1);

		writer.writeVarSInt(0b1111_1111111_1111111_1111111_1111110);
		writer.writeVarSInt(0b1111_1111111_1111111_1111110_1111111);
		writer.writeVarSInt(0b1111_1111111_1111110_1111111_1111111);
		writer.writeVarSInt(0b1111_1111110_1111111_1111111_1111111);
		writer.writeVarSInt(0b1110_1111111_1111111_1111111_1111111);
		writer.writeVarSInt(0b1010_1111111_1111111_1111111_1111111);
		writer.writeVarSInt(0b0110_1111111_1111111_1111111_1111111);
	}

	/**
	 * Test method for {@link com.is2t.giml.binary.GIMLWriterOutputStream#writeVarLong(long)}.
	 */
	@Test
	public void testWriteVarLong() {
		writeVarLong();
		byte[] expectedData = new byte[]{
				// 0
				0,
				//1
				(byte)0b00000010,
				//-1
				(byte)0b11111110,
				//Long.MAX_VALUE
				(byte)0b00000001, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111110,
				//Long.MAX_VALUE-1
				(byte)0b00000001, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111111, (byte)0b11111100,
				//Long.MIN_VALUE
				(byte)0b11111111, (byte)0b00000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000000,
				//Long.MIN_VALUE+1
				(byte)0b11111111, (byte)0b00000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000001, (byte)0b00000010,
		};

		testDataContent("writeVarLong", expectedData);
	}

	public void writeVarLong() {
		writer.writeVarLong(0);
		writer.writeVarLong(1);
		writer.writeVarLong(-1);
		writer.writeVarLong(Long.MAX_VALUE);
		writer.writeVarLong(Long.MAX_VALUE-1);
		writer.writeVarLong(Long.MIN_VALUE);
		writer.writeVarLong(Long.MIN_VALUE+1);
	}

	/**
	 * Test method for {@link com.is2t.giml.binary.GIMLWriterCompacted#writeBoolean(boolean)}.
	 */
	@Test
	public void testWriteBoolean() {
		writeBoolean();

		byte[] expectedData = new byte[]{ 1, 0};
		testDataContent("writeBoolean", expectedData);
	}

	public void writeBoolean() {
		writer.writeBoolean(true);
		writer.writeBoolean(false);
	}

	/**
	 * Test method for {@link com.is2t.giml.binary.GIMLWriterCompacted#writeFloat(float)}.
	 */
	@Test
	public void testWriteFloat() {
		float value = writeFloat();

		int bits = Float.floatToIntBits(value);
		byte[] expectedData = new byte[]{
				(byte)(bits>>24),
				(byte)(bits>>16),
				(byte)(bits>>8),
				(byte)(bits>>0),

		};
		testDataContent("writeFloat", expectedData);
	}

	public float writeFloat() {
		float value = 1.03245f;
		writer.writeFloat(value);
		return value;
	}

	/**
	 * Test method for {@link com.is2t.giml.binary.GIMLWriterCompacted#writeDouble(double)}.
	 */
	@Test
	public void testWriteDouble() {
		double value = writeDouble();

		long bits = Double.doubleToLongBits(value);
		byte[] expectedData = new byte[]{
				(byte)(bits>>56),
				(byte)(bits>>48),
				(byte)(bits>>40),
				(byte)(bits>>32),
				(byte)(bits>>24),
				(byte)(bits>>16),
				(byte)(bits>>8),
				(byte)(bits>>0),

		};
		testDataContent("writeDouble", expectedData);
	}

	public double writeDouble() {
		double value = 1.03245d;
		writer.writeDouble(value);
		return value;
	}

	/**
	 * Test method for {@link com.is2t.giml.binary.GIMLWriterCompacted#writeNLSStringId(int)}.
	 */
	@Test
	public void testWriteNLSStringId() {
		writeNLSStringId();

		byte[] expectedData = new byte[]{
				//0
				0b00000010,
				//54
				(byte)0b11011010,
				//63
				(byte)0b11111110,
				//64
				0b00000011, 0b00000010,

		};
		testDataContent("writeNLSStringId", expectedData);
	}

	public void writeNLSStringId() {
		int stringId;

		stringId = 0;
		writer.writeNLSStringId(stringId);

		stringId = 54;
		writer.writeNLSStringId(stringId);

		stringId = 63;
		writer.writeNLSStringId(stringId);

		stringId = 64;
		writer.writeNLSStringId(stringId);
	}

	/**
	 * Test method for {@link com.is2t.giml.binary.GIMLWriterCompacted#writeNonNLSString(java.lang.String)}.
	 */
	@Test
	public void testWriteNonNLSString() {
		String str1 = "yahoo!";
		String str2 = "yop";
		String str3 = "This is a very long string. This is a very long string. This is a very long string. This is a very long string. This is a very long string. ";
		writeNonNLSString(str1, str2, str3);

		try {
			writer.complete(0);
			byte[] constantPool = getConstantPool();
			byte[] data = getData();

			assertEquals(4, data.length);

			testNonNLSString(str1, 1+str1.length(), data, 0, constantPool);
			testNonNLSString(str2, 1+str2.length(), data, 1, constantPool);
			testNonNLSString(str1, 1+str1.length(), data, 2, constantPool);
			testNonNLSString(str3, 2+str3.length(), data, 3, constantPool);

			assertEquals("same string id for 2 references to the same string", data[0], data[2]);

		} catch (IOException e) {
			fail("writeNonNLSString: "+e.getMessage());
		}
	}

	public void writeNonNLSString(String str1, String str2, String str3) {
		writer.writeNonNLSString(str1);
		writer.writeNonNLSString(str2);
		writer.writeNonNLSString(str1);
		writer.writeNonNLSString(str3);
	}

	/**
	 * Test method for {@link com.is2t.giml.binary.GIMLWriterCompacted#writeNonNLSString(java.lang.String)} with UTF-8 string.
	 */
	@Test
	public void testWriteNonNLSUTF8String() {
		String str = writeNonNLSUTF8String();

		try {
			writer.complete(0);
			byte[] constantPool = getConstantPool();
			byte[] data = getData();

			assertEquals(1, data.length);

			testNonNLSString(str, str.getBytes("UTF-8").length+1, data, 0, constantPool);

		} catch (IOException e) {
			fail("writeNonNLSUTF8String: "+e.getMessage());
		}
	}

	public String writeNonNLSUTF8String() {
		String str = "abcdefg\u0178hijkl";
		writer.writeNonNLSString(str);
		return str;
	}

	/**
	 * Test method for {@link com.is2t.giml.binary.GIMLWriterCompacted#markPosition(java.lang.String)} with duplicated ID.
	 */
	@Test
	public void testMarkPositionDuplicated() {
		int identifier = 0;
		writer.writeVarUInt(42);
		writer.markPosition(identifier);
		writer.writeVarUInt(92);

		try{
			writer.markPosition(identifier);
			fail("duplicated ID must raise an exception");
		}
		catch(IllegalArgumentException e){
			// OK -> duplicated ID detected
		}
	}

	/**
	 * Test method for {@link com.is2t.giml.binary.GIMLWriterCompacted#markPosition(java.lang.String)}.
	 */
	@Test
	public void testMarkPositionSimple() {
		final int identifier = 0;
		writer.writeVarUInt(42);
		writer.markPosition(identifier);
		writer.writeVarUInt(92);

		String message = "testMarkPositionSimple";
		try{
			writer.complete(0);

			final int objectOffset = 1;

			byte[] expectedMap = new byte[]{
					//tableLength: 1
					0x0, 0x1,
					//bucketsTable
					0x0, 0x4,
					//bucket
					0x1, 0x0, 0x0, (byte) identifier, 0x0, 0x0, (byte) objectOffset,

			};
			// Get data
			byte[] objectMap = getObjectMap();

			// Compare data
			assertArrayEquals(message, expectedMap, objectMap);
		}catch (IOException e) {
			fail(message+": "+e.getMessage());
		}

	}



	/** Test that the data of the writer is equals to the given data */
	private void testDataContent(String message, byte[] expectedDataContent){
		try {
			writer.complete(0);

			// Get data
			byte[] data = getData();

			// Compare data
			assertArrayEquals(message, expectedDataContent, data);

		} catch (IOException e) {
			fail(message+": "+e.getMessage());
		}

	}

	private void testNonNLSString(String expected, int expectedEncodedSize, byte[] data, int offsetInData, byte[] constantPool){
		int stringId = data[offsetInData] >> 1;
		assertTrue("testNonNLSString internationalized bit must be 0", (stringId & 0x1) == 0);
		int stringOffset = stringId >> 1;

		StringEncoder encoder = new StringEncoder();
		byte[] encodedString = new byte[expectedEncodedSize];
		System.arraycopy(constantPool, stringOffset, encodedString, 0, expectedEncodedSize);
		try {
			String string = encoder.decodeString(encodedString);
			assertEquals("testNonNLSString", expected, string);
		} catch (IOException e) {
			fail("testNonNLSString: "+e.getMessage());
		}
	}

	private byte[] getData() throws IOException{
		// Get data start offset
		byte[] content = out.toByteArray();
		int dataOffset = getDataOffset(content);
		int dataLength = getDataLength(content);

		// Get data
		byte[] data = new byte[dataLength];
		System.arraycopy(content, dataOffset, data, 0, dataLength);

		return data;
	}

	private byte[] getConstantPool() throws IOException{
		// Get constant pool start offset
		byte[] content = out.toByteArray();
		int constantPoolOffset = getConstantPoolOffset(content);
		int constantPoolLength = getConstantPoolLength(content);

		// Get data
		byte[] constantPool = new byte[constantPoolLength];
		System.arraycopy(content, constantPoolOffset, constantPool, 0, constantPoolLength);

		return constantPool;
	}

	private byte[] getObjectMap() throws IOException{
		// Get data start offset
		byte[] content = out.toByteArray();
		int objectMapOffset = HEADER_SIZE;
		int objectMapLength = getDataOffset(content) - objectMapOffset;

		// Get data
		byte[] objectMap = new byte[objectMapLength];
		System.arraycopy(content, objectMapOffset, objectMap, 0, objectMapLength);

		return objectMap;
	}

	private int getDataOffset(byte[] content) throws IOException{
		try(DataInputStream dis = new DataInputStream(new ByteArrayInputStream(content))){
			// skip binary id
			dis.readLong();
			// skip constant pool offset
			dis.readUnsignedShort();
			return dis.readUnsignedShort();
		}
	}

	private int getDataLength(byte[] content) throws IOException{
		return getConstantPoolOffset(content) - getDataOffset(content);
	}

	private int getConstantPoolOffset(byte[] content) throws IOException{
		try(DataInputStream dis = new DataInputStream(new ByteArrayInputStream(content))){
			// skip binary id
			dis.readLong();
			// read constant pool offset
			return dis.readUnsignedShort();
		}
	}

	private int getConstantPoolLength(byte[] content) throws IOException{
		return content.length - getConstantPoolOffset(content);
	}

	public void completeWriter(long binaryId) throws IOException {
		this.writer.complete(binaryId);
	}

}
