/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.cssgenerator.selector;

import java.io.StringWriter;
import java.util.Hashtable;
import java.util.Map;

import org.w3c.css.sac.Condition;
import org.w3c.css.sac.DescendantSelector;

import com.steadystate.css.parser.selectors.ChildSelectorImpl;

/**
 *
 */
public class ChildCombinator implements Selector {

	private static final String CHILD_COMBINATOR_SELECTOR_VAR_PREFIX = "childCombi_";
	private final Map<String, String> childCombinatorSelectorVars = new Hashtable<String, String>();

	@Override
	public String generate(Condition c, StringWriter writer) throws IllegalArgumentException {
		throw new RuntimeException();
	}

	@Override
	public String generate(org.w3c.css.sac.Selector s, StringWriter writer) throws IllegalArgumentException {
		DescendantSelector descendantSelector = this.get(s);

		String ancestorSelectorVar = SelectorFactory.create(descendantSelector.getAncestorSelector(), writer);
		String simpleSelectorVar = SelectorFactory.create(descendantSelector.getSimpleSelector(), writer);
		StringBuilder childCombinatorSelector = new StringBuilder(ancestorSelectorVar).append(simpleSelectorVar);
		String childCombinatorSelectorVar = this.childCombinatorSelectorVars.get(childCombinatorSelector.toString());
		if (childCombinatorSelectorVar != null) {
			return childCombinatorSelectorVar;
		}
		StringBuilder childCombinatorSelectorVarSb = new StringBuilder(CHILD_COMBINATOR_SELECTOR_VAR_PREFIX)
				.append(ancestorSelectorVar).append("_").append(simpleSelectorVar);
		writer.append(this.getClassName()).append(" ").append(childCombinatorSelectorVarSb);
		writer.append(" = new ");
		writer.append(this.getClassName()).append('(');
		writer.append(ancestorSelectorVar);
		writer.append(", ");
		writer.append(simpleSelectorVar);
		writer.append(");\n");
		this.childCombinatorSelectorVars.put(childCombinatorSelector.toString(),
				childCombinatorSelectorVarSb.toString());
		return childCombinatorSelectorVarSb.toString();

	}

	private DescendantSelector get(org.w3c.css.sac.Selector s) {
		if (!this.matches(s)) {
			throw new IllegalArgumentException("Given selector is not a DescendantSelector");
		}

		return (DescendantSelector) s;
	}

	@Override
	public boolean matches(Condition condition) {
		return false;
	}

	@Override
	public String getClassName() {
		return "ej.style.selector.combinator.ChildCombinator";
	}

	@Override
	public boolean matches(org.w3c.css.sac.Selector selector) {
		return selector instanceof ChildSelectorImpl;
	}

}
