/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.cssgenerator.selector;

import java.io.StringWriter;
import java.util.Hashtable;
import java.util.Map;

import org.w3c.css.sac.Condition;
import org.w3c.css.sac.ElementSelector;

import com.is2t.giml.description.Description;
import com.is2t.giml.description.DescriptionFactory;
import com.steadystate.css.parser.selectors.ElementSelectorImpl;

/**
 *
 */
public class TypeSelector implements Selector {

	private static final String TYPE_SELECTOR_VAR_PREFIX = "type_";
	private final Map<String, String> typeSelectorVars = new Hashtable<String, String>();

	@Override
	public String generate(Condition c, StringWriter writer) throws IllegalArgumentException {
		throw new RuntimeException();
	}

	@Override
	public String generate(org.w3c.css.sac.Selector s, StringWriter writer) {
		String className = getDescription(s).getClassName();
		if (this.typeSelectorVars.containsKey(className)) {
			return this.typeSelectorVars.get(className);
		}
		StringBuilder typeSelectorVar = new StringBuilder(TYPE_SELECTOR_VAR_PREFIX)
				.append(this.typeSelectorVars.size());
		writer.append(this.getClassName()).append(" ").append(typeSelectorVar).append(" = ");
		writer.append("new ");
		writer.append(this.getClassName()).append("(");
		writer.append(className);
		writer.append(".class);\n");
		this.typeSelectorVars.put(className, typeSelectorVar.toString());
		return typeSelectorVar.toString();
	}

	private Description getDescription(org.w3c.css.sac.Selector s) {
		ElementSelector elementSelector = this.get(s);
		String elementTagName = elementSelector.getLocalName();
		Description description = DescriptionFactory.getDescription(elementTagName);
		if (description == null) {
			throw new IllegalArgumentException(String.format("Unknown element %s", elementTagName));
		}
		if (description.getClassName() == null) {
			throw new IllegalArgumentException(String.format("Element %s has no class", elementTagName));
		}
		return description;
	}

	private ElementSelector get(org.w3c.css.sac.Selector s) {
		if (!this.matches(s)) {
			throw new IllegalArgumentException("Given selector is not an ElementSelector");
		}

		return (ElementSelector) s;
	}

	@Override
	public boolean matches(Condition condition) {
		return false;
	}

	@Override
	public String getClassName() {
		return "ej.style.selector.TypeOrSubtypeSelector";
	}

	@Override
	public boolean matches(org.w3c.css.sac.Selector selector) {
		if (!(selector instanceof ElementSelectorImpl)) {
			return false;
		}

		ElementSelectorImpl s = (ElementSelectorImpl) selector;
		if (s.getLocalName() == null || "*".equalsIgnoreCase(s.getLocalName())) {
			return false;
		}

		return true;
	}

}
