/*
 * Java
 *
 * Copyright 2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.generator.js;

import java.util.List;

import com.is2t.giml.generator.GIMLGenerator;
import com.is2t.giml.jsgenerator.JavaScriptSourceToMicroEJSourceGenerator;
import com.is2t.giml.jsgenerator.UnsupportedJavaScriptSyntax;
import com.is2t.giml.jsgenerator.global.GlobalObject;
import com.is2t.java.nodes.Expression;
import com.is2t.java.nodes.Statement;
import com.is2t.java.nodes.expressions.IntegerLiteral;
import com.is2t.java.nodes.expressions.StringLiteral;

import jdk.nashorn.internal.ir.CallNode;

/**
 * <code>giml</code> global object
 */
public class GlobalObjectGIML implements GlobalObject {

	private final GIMLGenerator gimlGenerator;

	public GlobalObjectGIML(GIMLGenerator gimlGenerator) {
		this.gimlGenerator = gimlGenerator;
	}

	@Override
	public String getGlobalName() {
		return "giml";
	}

	@Override
	public Statement generateInitialization(JavaScriptSourceToMicroEJSourceGenerator gen) {
		return gen.generateEmptyBlockStatement();
	}

	@Override
	public Expression generateCallExpression(JavaScriptSourceToMicroEJSourceGenerator gen, CallNode call,
			String functionName) {

		// giml.ref('myIdentifier')
		if (functionName.equals("ref")) {
			List<jdk.nashorn.internal.ir.Expression> args = call.getArgs();
			if (args.size() != 1) {
				throw new UnsupportedJavaScriptSyntax(call);
			}
			args.get(0).accept(gen);
			Expression javaArg = gen.popExpression();
			if (!(javaArg instanceof StringLiteral)) {
				throw new UnsupportedJavaScriptSyntax(call);
			}
			String identifierAsStr = new String(((StringLiteral) javaArg).value);
			// TODO add a warning undefined identifier
			// if not found, the code is generated correctly (-1 is pushed), and an error
			// will be thrown at runtime if the code is executed
			return new IntegerLiteral(gimlGenerator.getRootIdentifier(identifierAsStr));
		} else {
			throw new UnsupportedJavaScriptSyntax(call);
		}
	}

}
