/*
 * Java
 *
 * Copyright 2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.jsgenerator.test;

import java.io.IOException;
import java.io.StringReader;
import java.util.List;

import org.junit.Assert;

import com.is2t.giml.classpath.ClasspathModel;
import com.is2t.giml.jsgenerator.JavaScriptGenerator;
import com.is2t.giml.jsgenerator.JavaScriptParser;
import com.is2t.giml.jsgenerator.JavaScriptSourcePreGenerator;
import com.is2t.giml.jsgenerator.JavaScriptSourceToMicroEJSourceGenerator;
import com.is2t.giml.jsgenerator.global.GlobalObjectConsole;
import com.is2t.java.JavaSourceToStringGenerator;
import com.is2t.java.nodes.JavaSourceNode;
import com.is2t.java.nodes.MethodDeclaration;

import jdk.nashorn.internal.ir.FunctionNode;
import jdk.nashorn.internal.ir.Statement;

/**
 *
 */
public class AbstractTest {

	static {
		if (!AbstractTest.class.desiredAssertionStatus()) {
			throw new AssertionError("Java asserts must be enabled in order to run this testsuite");
		}
	}

	/**
	 *
	 */
	public AbstractTest() {
		super();
	}

	/**
	 * Test JS/Java conversion (expected identical code)
	 *
	 * @param identCode
	 *            the JS code whiwh is also the expected Java code
	 */
	protected void testJSSourceToMicroEJSource(String identCode) {
		testJSSourceToMicroEJSource(identCode, identCode);
	}

	protected void testJSSourceToMicroEJSource(String jsCode, String expectedJavaCode) {
		JavaScriptParser parser = new JavaScriptParser();
		List<Statement> statements;
		try {
			statements = parser.parseInlinedCode(new StringReader(jsCode));
		} catch (IOException e) {
			throw new AssertionError(e);
		}
		Assert.assertEquals(statements.size(), 1);

		JavaScriptGenerator jsGen = new JavaScriptGenerator(new ClasspathModel());
		jsGen.addGlobalObject(new GlobalObjectConsole());
		JavaScriptSourcePreGenerator jsSourcePreGenerator = new JavaScriptSourcePreGenerator();

		JavaScriptSourceToMicroEJSourceGenerator jsToJavaGen = new JavaScriptSourceToMicroEJSourceGenerator(jsGen,
				jsSourcePreGenerator);
		JavaSourceNode javaNode = jsToJavaGen.generate(statements.get(0));
		if(javaNode == null) {
			// a function - wrap to a MethodDeclaration before String generation
			List<FunctionNode> functions = jsSourcePreGenerator.getFunctions();
			Assert.assertEquals(functions.size(), 1);
			FunctionNode functionNode = functions.get(0);
			MethodDeclaration method = jsToJavaGen.generate(functionNode);
			javaNode = method;
		}
		else {
			// a statement / generate as-is
		}

		JavaSourceToStringGenerator gen = new JavaSourceToStringGenerator();
		String generatedJavaCode = gen.toString(javaNode);

		String canonizedGeneratedJavaCode = removeWhitespaces(generatedJavaCode);

		Assert.assertEquals(expectedJavaCode, canonizedGeneratedJavaCode);
	}

	/**
	 * Canonize a string by removing all whitespaces and line delimiters
	 *
	 * @param str
	 *            the string to be canonized
	 * @return a canonized single line of string
	 */
	public static String removeWhitespaces(String str) {
		StringBuilder resultVect = new StringBuilder();
		for (char c : str.toCharArray()) {
			if (c <= 0x20) { // the space character of below (same spec as trim())
				continue;
			}
			resultVect.append(c);
		}
		return resultVect.toString();
	}

}