/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.xml.parser.test.xmlparser;

import java.io.File;
import java.io.FileNotFoundException;
import java.net.URL;

import org.apache.commons.io.FileUtils;
import org.junit.Test;

import com.is2t.giml.xml.parser.XMLParser;
import com.is2t.giml.xml.parser.exception.XMLParserException;
import com.is2t.giml.xml.parser.exception.XMLParserMalformedTemplateException;
import com.is2t.giml.xml.parser.test.AbstractTest;

/**
 * Test cases of {@link XMLParser} on invalid templates.
 */
public class XMLParserInvalidDocumentTemplatesTest extends AbstractTest {

	// We do not test behavior of templates, this is done in XML Model library.

	private static final String RESOURCE_FOLDER_PATH = "/com/is2t/giml/xml/parser/test/xmlparser/templates/invalid/";

	private static final String PARSER_OPTION_XML_EXTENSION = ".xml";
	private static final String PARSER_OPTION_DOCUMENT_ID_ATTRIBUTE_NAME = "name";
	private static final String PARSER_OPTION_IMPORT_TAGNAME = "import";
	private static final String PARSER_OPTION_IMPORT_ATTRIBUTE_NAME = "name";
	private static final boolean PARSER_OPTION_HANDLE_IMPORTS = true;
	private static final String PARSER_OPTION_TEMPLATE_TAGNAME = "template";
	private static final String PARSER_OPTION_TEMPLATE_ATTRIBUTE_NAME = "name";
	private static final String PARSER_OPTION_TEMPLATE_PARAMETER_PREFIX = "{{";
	private static final String PARSER_OPTION_TEMPLATE_PARAMETER_SUFFIX = "}}";
	private static final boolean PARSER_OPTION_HANDLE_TEMPLATES = true;

	private static File getFileFromJarResource(String resourceName) {
		String resourcePath = RESOURCE_FOLDER_PATH + resourceName;
		URL fileURL = AbstractTest.class.getResource(resourcePath);
		return FileUtils.toFile(fileURL);
	}

	private XMLParser createXMLParser(File rootFile) throws FileNotFoundException {
		XMLParser parser = new XMLParser(rootFile);
		parser.getOptions().setXMLFileExtension(PARSER_OPTION_XML_EXTENSION);
		parser.getOptions().setDocumentIdAttributeName(PARSER_OPTION_DOCUMENT_ID_ATTRIBUTE_NAME);
		parser.getOptions().setImportTagName(PARSER_OPTION_IMPORT_TAGNAME);
		parser.getOptions().setImportTagAttributeName(PARSER_OPTION_IMPORT_ATTRIBUTE_NAME);
		parser.getOptions().setHandleImports(PARSER_OPTION_HANDLE_IMPORTS);
		parser.getOptions().setTemplateTagName(PARSER_OPTION_TEMPLATE_TAGNAME);
		parser.getOptions().setTemplateTagAttributeName(PARSER_OPTION_TEMPLATE_ATTRIBUTE_NAME);
		parser.getOptions().setTemplateParameterPrefix(PARSER_OPTION_TEMPLATE_PARAMETER_PREFIX);
		parser.getOptions().setTemplateParameterSuffix(PARSER_OPTION_TEMPLATE_PARAMETER_SUFFIX);
		parser.getOptions().setHandleTemplates(PARSER_OPTION_HANDLE_TEMPLATES);
		return parser;
	}

	@Test(expected = XMLParserMalformedTemplateException.class)
	public void testMultiRootTemplate() throws XMLParserException, FileNotFoundException {
		createXMLParser(getFileFromJarResource("multiroot/multiroot.xml")).parse();
	}

	@Test(expected = IllegalArgumentException.class)
	public void testMissingParameterInInvocation() throws XMLParserException, FileNotFoundException {
		createXMLParser(getFileFromJarResource("missingparam/missingparam.xml")).parse().compact().resolveTemplates();
	}

	@Test(expected = IllegalArgumentException.class)
	public void testTooMuchParametersInInvocation() throws XMLParserException, FileNotFoundException {
		createXMLParser(getFileFromJarResource("toomuchparam/toomuchparam.xml")).parse().compact().resolveTemplates();
	}

	@Test(expected = XMLParserMalformedTemplateException.class)
	public void testTemplateAlreadyExists() throws XMLParserException, FileNotFoundException {
		createXMLParser(getFileFromJarResource("alreadyexists/alreadyexists.xml")).parse().compact().resolveTemplates();
	}

}
