/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.xml.parser.test.xmlparseroptions;

import org.junit.Test;

import com.is2t.giml.xml.parser.XMLParserConstants;
import com.is2t.giml.xml.parser.XMLParserOptions;

import junit.framework.Assert;

/**
 * Unit tests for {@link XMLParserOptions} class.
 */
public class XMLParserOptionsTest {

	private static final String NEW_EXTENSION_1 = "ext";
	private static final String NEW_EXTENSION_2 = ".ext";
	private static final String NEW_DOCUMENT_ID = "new-document-id";
	private static final String NEW_IMPORT_TAG = "new-import-tag";
	private static final String NEW_IMPORT_TAG_ATTRIBUTE_NAME = "new-import-tag-attr-name";
	private static final String NEW_TEMPLATE_TAG = "new-template-tag";
	private static final String NEW_TEMPLATE_TAG_ATTRIBUTE_NAME = "new-template-tag-attr-name";
	private static final String NEW_TEMPLATE_PARAMETER_PREFIX = "[[prefix[[";
	private static final String NEW_TEMPLATE_PARAMETER_SUFFIX = "]]suffix]]";
	private static final String NEW_TEMPLATE_CONTENT_TAG = "new-content";
	private static final String NEW_TEMPLATE_CONTENT_TAG_ATTRIBUTE_NAME = "new-content-attr";
	private static final String NEW_TEMPLATE_CONTENT_TAG_INSTANTIATION_ATTRIBUTE_NAME = "new-content-attr-i";

	@Test
	public void testDefaultValues() {
		XMLParserOptions options = new XMLParserOptions();
		Assert.assertEquals(options.getXMLFileExtension(), XMLParserConstants.DEFAULT_XML_FILE_EXTENSION);
		Assert.assertEquals(options.getDocumentIdAttributeName(),
				XMLParserConstants.DEFAULT_DOCUMENT_ID_ATTRIBUTE_NAME);
		Assert.assertEquals(options.handleImports(), XMLParserConstants.DEFAULT_HANDLE_IMPORTS);
		Assert.assertEquals(options.getImportTagName(), XMLParserConstants.DEFAULT_IMPORT_DOCUMENT_TAGNAME);
		Assert.assertEquals(options.getImportTagAttributeName(), XMLParserConstants.DEFAULT_IMPORT_NAME_ATTRIBUTE_NAME);
		Assert.assertEquals(options.handleTemplates(), XMLParserConstants.DEFAULT_HANDLE_TEMPLATES);
		Assert.assertEquals(options.getTemplateTagName(), XMLParserConstants.DEFAULT_TEMPLATE_TAGNAME);
		Assert.assertEquals(options.getTemplateTagAttributeName(),
				XMLParserConstants.DEFAULT_TEMPLATE_NAME_ATTRIBUTE_NAME);
		Assert.assertEquals(options.getTemplateParameterPrefix(), XMLParserConstants.DEFAULT_TEMPLATE_PARAMETER_PREFIX);
		Assert.assertEquals(options.getTemplateParameterSuffix(), XMLParserConstants.DEFAULT_TEMPLATE_PARAMETER_SUFFIX);
		Assert.assertEquals(options.getTemplateContentTagName(), XMLParserConstants.DEFAULT_TEMPLATE_CONTENT_TAGNAME);
		Assert.assertEquals(options.getTemplateContentTagAttributeName(),
				XMLParserConstants.DEFAULT_TEMPLATE_CONTENT_NAME_ATTRIBUTE_NAME);
		Assert.assertEquals(options.getTemplateContentTagInstantiationAttributeName(),
				XMLParserConstants.DEFAULT_TEMPLATE_CONTENT_INSTANTIATION_NAME_ATTRIBUTE_NAME);
	}

	@Test
	public void testEquals() {
		XMLParserOptions xmlParserOptions1 = new XMLParserOptions();
		XMLParserOptions xmlParserOptions2 = new XMLParserOptions();
		XMLParserOptions xmlParserOptions3 = new XMLParserOptions();
		xmlParserOptions3.setXMLFileExtension("other");

		Assert.assertFalse(xmlParserOptions1.equals(null));
		Assert.assertFalse(xmlParserOptions1.equals(new String()));
		Assert.assertFalse(xmlParserOptions1.equals(xmlParserOptions3));
		Assert.assertFalse(xmlParserOptions2.equals(xmlParserOptions3));

		Assert.assertTrue(xmlParserOptions1.equals(xmlParserOptions2));
	}

	/* Test Extension */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueXMLExtension() {
		new XMLParserOptions().setXMLFileExtension(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueXMLExtension() {
		new XMLParserOptions().setXMLFileExtension("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesXMLExtension() {
		new XMLParserOptions().setXMLFileExtension(".ext t");
	}

	@Test
	public void setValidValueXMLExtension() {
		XMLParserOptions options = new XMLParserOptions();
		options.setXMLFileExtension(NEW_EXTENSION_1);
		Assert.assertEquals(options.getXMLFileExtension(), NEW_EXTENSION_1);
		options.setXMLFileExtension(NEW_EXTENSION_2);
		Assert.assertEquals(options.getXMLFileExtension(), NEW_EXTENSION_2);
	}

	/* Test Document ID attribute */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueDocumentId() {
		new XMLParserOptions().setDocumentIdAttributeName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueDocumentId() {
		new XMLParserOptions().setDocumentIdAttributeName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesDocumentId() {
		new XMLParserOptions().setDocumentIdAttributeName("with space");
	}

	@Test
	public void setValidValueDocumentId() {
		XMLParserOptions options = new XMLParserOptions();
		options.setDocumentIdAttributeName(NEW_DOCUMENT_ID);
		Assert.assertEquals(options.getDocumentIdAttributeName(), NEW_DOCUMENT_ID);
	}

	/* Test handle imports */

	@Test
	public void setValidValuesHandleImports() {
		XMLParserOptions options = new XMLParserOptions();
		options.setHandleImports(true);
		Assert.assertTrue(options.handleImports());
		options.setHandleImports(false);
		Assert.assertFalse(options.handleImports());
	}

	/* Test Import Tag name */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueImportTag() {
		new XMLParserOptions().setImportTagName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueImportTag() {
		new XMLParserOptions().setImportTagName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesImportTag() {
		new XMLParserOptions().setImportTagName("with space");
	}

	@Test
	public void setValidValueImportTag() {
		XMLParserOptions options = new XMLParserOptions();
		options.setImportTagName(NEW_IMPORT_TAG);
		Assert.assertEquals(options.getImportTagName(), NEW_IMPORT_TAG);
	}

	/* Test Import Attribute Name */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueImportAttributeName() {
		new XMLParserOptions().setImportTagAttributeName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueImportAttributeName() {
		new XMLParserOptions().setImportTagAttributeName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesImportAttributeName() {
		new XMLParserOptions().setImportTagAttributeName("with space");
	}

	@Test
	public void setValidValueImportAttributeName() {
		XMLParserOptions options = new XMLParserOptions();
		options.setImportTagAttributeName(NEW_IMPORT_TAG_ATTRIBUTE_NAME);
		Assert.assertEquals(options.getImportTagAttributeName(), NEW_IMPORT_TAG_ATTRIBUTE_NAME);
	}

	/* Test handle templates */

	@Test
	public void setValidValuesHandleTemplates() {
		XMLParserOptions options = new XMLParserOptions();
		options.setHandleTemplates(true);
		Assert.assertTrue(options.handleTemplates());
		options.setHandleTemplates(false);
		Assert.assertFalse(options.handleTemplates());
	}

	/* Test Template Tag name */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueTemplateTag() {
		new XMLParserOptions().setTemplateTagName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueTemplateTag() {
		new XMLParserOptions().setTemplateTagName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesTemplateTag() {
		new XMLParserOptions().setTemplateTagName("with space");
	}

	@Test
	public void setValidValueTemplateTag() {
		XMLParserOptions options = new XMLParserOptions();
		options.setTemplateTagName(NEW_TEMPLATE_TAG);
		Assert.assertEquals(options.getTemplateTagName(), NEW_TEMPLATE_TAG);
	}

	/* Test Template Attribute Name */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueTemplateAttributeName() {
		new XMLParserOptions().setTemplateTagAttributeName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueTemplateAttributeName() {
		new XMLParserOptions().setTemplateTagAttributeName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesTemplateAttributeName() {
		new XMLParserOptions().setTemplateTagAttributeName("with space");
	}

	@Test
	public void setValidValueTemplateAttributeName() {
		XMLParserOptions options = new XMLParserOptions();
		options.setTemplateTagAttributeName(NEW_TEMPLATE_TAG_ATTRIBUTE_NAME);
		Assert.assertEquals(options.getTemplateTagAttributeName(), NEW_TEMPLATE_TAG_ATTRIBUTE_NAME);
	}

	/* Test Template parameter prefix */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueTemplateParameterPrefix() {
		new XMLParserOptions().setTemplateParameterPrefix(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueTemplateParameterPrefix() {
		new XMLParserOptions().setTemplateParameterPrefix("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesTemplateParameterPrefix() {
		new XMLParserOptions().setTemplateParameterPrefix("with space");
	}

	@Test
	public void setValidValueTemplateParameterPrefix() {
		XMLParserOptions options = new XMLParserOptions();
		options.setTemplateParameterPrefix(NEW_TEMPLATE_PARAMETER_PREFIX);
		Assert.assertEquals(options.getTemplateParameterPrefix(), NEW_TEMPLATE_PARAMETER_PREFIX);
	}

	/* Test Template parameter suffix */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueTemplateParameterSuffix() {
		new XMLParserOptions().setTemplateParameterSuffix(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueTemplateParameterSuffix() {
		new XMLParserOptions().setTemplateParameterSuffix("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesTemplateParameterSuffix() {
		new XMLParserOptions().setTemplateParameterSuffix("with space");
	}

	@Test
	public void setValidValueTemplateParameterSuffix() {
		XMLParserOptions options = new XMLParserOptions();
		options.setTemplateParameterSuffix(NEW_TEMPLATE_PARAMETER_SUFFIX);
		Assert.assertEquals(options.getTemplateParameterSuffix(), NEW_TEMPLATE_PARAMETER_SUFFIX);
	}

	/* Test Template content tag name */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueTemplateContentTag() {
		new XMLParserOptions().setTemplateContentTagName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueTemplateContentTag() {
		new XMLParserOptions().setTemplateContentTagName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesTemplateContentTag() {
		new XMLParserOptions().setTemplateContentTagName("with space");
	}

	@Test
	public void setValidValueTemplateContentTag() {
		XMLParserOptions options = new XMLParserOptions();
		options.setTemplateContentTagName(NEW_TEMPLATE_CONTENT_TAG);
		Assert.assertEquals(options.getTemplateContentTagName(), NEW_TEMPLATE_CONTENT_TAG);
	}

	/* Test Template content tag attribute name */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueTemplateContentTagAttributeName() {
		new XMLParserOptions().setTemplateContentTagAttributeName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueTemplateContentTagAttributeName() {
		new XMLParserOptions().setTemplateContentTagAttributeName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesTemplateContentTagAttributeName() {
		new XMLParserOptions().setTemplateContentTagAttributeName("with space");
	}

	@Test
	public void setValidValueTemplateContentTagAttributeName() {
		XMLParserOptions options = new XMLParserOptions();
		options.setTemplateContentTagAttributeName(NEW_TEMPLATE_CONTENT_TAG_ATTRIBUTE_NAME);
		Assert.assertEquals(options.getTemplateContentTagAttributeName(), NEW_TEMPLATE_CONTENT_TAG_ATTRIBUTE_NAME);
	}

	/* Test Template content tag attribute name */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueTemplateContentTagInstantiationAttributeName() {
		new XMLParserOptions().setTemplateContentTagInstantiationAttributeName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueTemplateContentTagInstantiationAttributeName() {
		new XMLParserOptions().setTemplateContentTagInstantiationAttributeName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesTemplateContentTagInstantiationAttributeName() {
		new XMLParserOptions().setTemplateContentTagInstantiationAttributeName("with space");
	}

	@Test
	public void setValidValueTemplateContentTagInstantiationAttributeName() {
		XMLParserOptions options = new XMLParserOptions();
		options.setTemplateContentTagInstantiationAttributeName(NEW_TEMPLATE_CONTENT_TAG_INSTANTIATION_ATTRIBUTE_NAME);
		Assert.assertEquals(options.getTemplateContentTagInstantiationAttributeName(),
				NEW_TEMPLATE_CONTENT_TAG_INSTANTIATION_ATTRIBUTE_NAME);
	}

}
