/*
 * Copyright (c) 2015, Oracle and/or its affiliates. All rights reserved.
 * Copyright (C) 2015-2020, IS2T. EDC compliance and optimizations.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.is2t.support.security.natives;

import com.is2t.support.security.SecurityErrorConstants;

import ej.sni.NativeException;

public class X509CertSupportNatives {

	/**
	 * Parses the given certificate to validate it and returns its encoded format type.
	 *
	 * @param cert
	 *            the certificate buffer
	 * @param off
	 *            the offset in the buffer at which the certificate content started
	 * @param len
	 *            the certificate content length * @param retry true if this operation is called again when the previous
	 *            call is not completed.
	 * @return the certificate encoded format type (0 if PEM format or 1 if DER format);
	 *         {@link SecurityErrorConstants#J_CERT_PARSE_ERROR} if the certificate is not valid or an error occurs.
	 */
	public static native int parse(byte[] cert, int off, int len);

	/**
	 *
	 * @param cert
	 * @param certLen
	 * @param keyData
	 * @param keyDataLength in parameter. Contains the length of keyData.
	 *
	 * @throws NativeException on error.
	 *
	 * @return the number of bytes copied into keyData
	 */
	public static native int getKeyData(byte[] cert, int certLen, byte[] keyData, int keyDataLength) throws NativeException;


	/**
	 * Returns the size of the keyData buffer that will be passed to {@link #getKeyData(byte[], int, byte[], int)}.
	 */
	public static native int getKeyDataSize();

	/**
	 *
	 * @param cert
	 * @param certLen
	 * @param principalData
	 * @param principalDataLength
	 *            in parameter. Contains the length of keyData.
	 * @param getIssuer
	 *
	 * @throws NativeException
	 *             on error.
	 *
	 *             return the number of bytes copied into principalData
	 */
	public static native int getX500PrincipalData(byte[] cert, int certLen, byte[] principalData,
			int principalDataLength, boolean getIssuer) throws NativeException;

	/**
	 * @param data
	 * @param i
	 * @param length
	 * @param keyBytes
	 * @param j
	 * @param length2
	 */
	public static native int verify(byte[] cert, int certLength, byte[] keyData, int keyDataLength)
			throws NativeException;
}
