<?xml version="1.0"?>
<!DOCTYPE foo [
	<!ENTITY style SYSTEM "stylesheet.css">
]>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
        xmlns:lxslt="http://xml.apache.org/xslt"
        xmlns:stringutils="xalan://org.apache.tools.ant.util.StringUtils">
<xsl:output method="html" indent="yes" encoding="UTF-8"
  doctype-public="-//W3C//DTD HTML 4.01 Transitional//EN" />
<xsl:decimal-format decimal-separator="." grouping-separator="," />
<!--
   Licensed to the Apache Software Foundation (ASF) under one or more
   contributor license agreements.  See the NOTICE file distributed with
   this work for additional information regarding copyright ownership.
   The ASF licenses this file to You under the Apache License, Version 2.0
   (the "License"); you may not use this file except in compliance with
   the License.  You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 -->

<xsl:param name="TITLE">Testsuite Results</xsl:param>
<xsl:param name="PRODUCT"/>

<!--

 Sample stylesheet to be used with Ant JUnitReport output.

 It creates a non-framed report that can be useful to send via
 e-mail or such.

-->
<xsl:template match="testsuites">
    
    <html class="report">
        <head>
            <title><xsl:value-of select="$PRODUCT"/> • <xsl:value-of select="$TITLE"/></title>
            <style type="text/css">
&style;
            </style>
        </head>
        <body class="report-body" id="top">
            <h1><xsl:value-of select="$TITLE"/>: <xsl:value-of select="$PRODUCT"/></h1>
            <!-- Summary part -->
            <xsl:call-template name="summary"/>
            <hr/>

            <!-- Package List part -->
            <xsl:call-template name="packagelist"/>
            <hr/>

            <!-- For each package create its part -->
            <xsl:call-template name="packages"/>
            <hr/>

            <!-- For each class create the  part -->
            <xsl:call-template name="classes"/>

        </body>
    </html>
</xsl:template>



    <!-- ================================================================== -->
    <!-- Write a list of all packages with an hyperlink to the anchor of    -->
    <!-- of the package name.                                               -->
    <!-- ================================================================== -->
    <xsl:template name="packagelist">
        <h2>Packages</h2>
        <p class="note">Note: package statistics are not computed recursively, they only sum up all of its testsuites numbers.</p>
        <table class="details">
            <xsl:call-template name="testsuite.test.header"/>
            <!-- list all packages recursively -->
            <xsl:for-each select="./testsuite[not(./@package = preceding-sibling::testsuite/@package)]">
                <xsl:sort select="@package"/>
                <xsl:variable name="testsuites-in-package" select="/testsuites/testsuite[./@package = current()/@package]"/>
                <xsl:variable name="testCount" select="sum($testsuites-in-package/@tests)"/>
                <xsl:variable name="errorCount" select="sum($testsuites-in-package/@errors)"/>
                <xsl:variable name="failureCount" select="sum($testsuites-in-package/@failures)"/>
                <xsl:variable name="timeCount" select="sum($testsuites-in-package/@time)"/>
                <xsl:variable name="ignoredCount" select="sum($testsuites-in-package/@ignored)"/>
                <xsl:variable name="triedAgainCount" select="sum($testsuites-in-package/@triedAgain)"/>
                <!-- write a summary for the package -->
                <tr>
                    <td><a href="#{@package}"><code><xsl:value-of select="@package"/></code></a></td>
                    <td><xsl:value-of select="$testCount"/></td>
                    <td>
                    <xsl:attribute name="class">
		                <xsl:if test="$errorCount &gt; 0">status error</xsl:if>
		            </xsl:attribute>
                    <xsl:value-of select="$errorCount"/></td>
                    <td>
                    <xsl:attribute name="class">
		                <xsl:if test="$failureCount &gt; 0">status failure</xsl:if>
		            </xsl:attribute>
                    <xsl:value-of select="$failureCount"/></td>
                    <td>
                    <xsl:attribute name="class">
		                <xsl:if test="$ignoredCount &gt; 0">status ignored</xsl:if>
		            </xsl:attribute>
                    <xsl:value-of select="$ignoredCount"/></td>
                    <td>
                    <xsl:attribute name="class">
		                <xsl:if test="$triedAgainCount &gt; 0">status triedAgain</xsl:if>
		            </xsl:attribute>
                    <xsl:value-of select="$triedAgainCount"/></td>
                    <td>
                    <xsl:call-template name="display-time">
                        <xsl:with-param name="value" select="$timeCount"/>
                    </xsl:call-template>
                    </td>
                    <td><xsl:value-of select="$testsuites-in-package/@timestamp"/></td>
                    <td><xsl:value-of select="$testsuites-in-package/@hostname"/></td>
                </tr>
            </xsl:for-each>
        </table>
    </xsl:template>


    <!-- ================================================================== -->
    <!-- Write a package level report                                       -->
    <!-- It creates a table with values from the document:                  -->
    <!-- Name | Tests | Errors | Failures | Time                            -->
    <!-- ================================================================== -->
    <xsl:template name="packages">
        <xsl:for-each select="/testsuites/testsuite[not(./@package = preceding-sibling::testsuite/@package)]">
            <xsl:sort select="@package"/>
                <h3 id="{@package}">Package <code><xsl:value-of select="@package"/></code></h3>

                <table class="details">
                    <xsl:call-template name="testsuite.test.header"/>

                    <!-- match the testsuites of this package -->
                    <xsl:apply-templates select="/testsuites/testsuite[./@package = current()/@package]" mode="print.test"/>
                </table>
                <a href="#top">Back to top</a>
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="classes">
        <xsl:for-each select="testsuite">
            <xsl:sort select="@name"/>
            <h3 id="{@name}">TestCase <xsl:value-of select="@name"/></h3>

            <table class="details">
              <xsl:call-template name="testcase.test.header"/>
              <!--
              test can even not be started at all (failure to load the class)
              so report the error directly
              -->
                <xsl:if test="./error">
                    <tr>
                        <td class="status error" colspan="4"><xsl:apply-templates select="./error"/></td>
                    </tr>
                </xsl:if>
                <xsl:apply-templates select="./testcase" mode="print.test"/>
            </table>

            <a href="#top">Back to top</a>
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="summary">
        <h2>Summary</h2>
        <xsl:variable name="testCount" select="sum(testsuite/@tests)"/>
        <xsl:variable name="errorCount" select="sum(testsuite/@errors)"/>
        <xsl:variable name="failureCount" select="sum(testsuite/@failures)"/>
        <xsl:variable name="timeCount" select="sum(testsuite/@time)"/>
        <xsl:variable name="ignoredCount" select="sum(testsuite/@ignored)"/>
        <xsl:variable name="triedAgainCount" select="sum(testsuite/@triedAgain)"/>
        <xsl:variable name="successRate" select="($testCount - $failureCount - $errorCount - $ignoredCount) div ($testCount - $ignoredCount)"/>
        <xsl:variable name="assertsOkCount" select="sum(testsuite[@ignored='0']/@asserts_ok)"/>
        <xsl:variable name="assertsNokCount" select="sum(testsuite[@ignored='0']/@asserts_nok)"/>
        <xsl:variable name="assertsCount" select="$assertsOkCount + $assertsNokCount"/>
        <xsl:variable name="assertsSuccessRate" select="($assertsCount - $assertsNokCount) div $assertsCount"/>
        <table class="details">
        <thead>
        <tr>
            <th>Tests</th>
            <th>Failures</th>
            <th>Errors</th>
            <th>Ignored</th>
            <th>Tried Again</th>
            <th>Success rate</th>
            <th>Time</th>
        </tr>
        </thead>
        <tr>
            <td><xsl:value-of select="$testCount"/></td>
            <td>
            <xsl:attribute name="class">
                <xsl:if test="$failureCount &gt; 0">status failure</xsl:if>
            </xsl:attribute>
            <xsl:value-of select="$failureCount"/></td>
            <td>
            <xsl:attribute name="class">
                <xsl:if test="$errorCount &gt; 0">status error</xsl:if>
            </xsl:attribute>
            <xsl:value-of select="$errorCount"/></td>
            <td>
            <xsl:attribute name="class">
                <xsl:if test="$ignoredCount &gt; 0">status ignored</xsl:if>
            </xsl:attribute>
            <xsl:value-of select="$ignoredCount"/></td>
            <td>
            <xsl:attribute name="class">
                <xsl:if test="$triedAgainCount &gt; 0">status triedAgain</xsl:if>
            </xsl:attribute>
            <xsl:value-of select="$triedAgainCount"/></td>
            <td>
	            <xsl:attribute name="class">
	                <xsl:if test="$successRate = 1">status success</xsl:if>
	            </xsl:attribute>
                <xsl:call-template name="display-percent">
                    <xsl:with-param name="value" select="$successRate"/>
                </xsl:call-template>
            </td>
            <td>
                <xsl:call-template name="display-time">
                    <xsl:with-param name="value" select="$timeCount"/>
                </xsl:call-template>
            </td>
        </tr>
        </table>
        
    	<table class="details">
        <thead>
        <tr>
            <th>Assertions</th>
            <th>Failures</th>
            <th>Success</th>
            <th>Success Rate</th>
        </tr>
        </thead>
        <tr>
            <td><xsl:value-of select="$assertsCount"/></td>
            <td>
            <xsl:attribute name="class">
                <xsl:if test="$assertsNokCount &gt; 0">status failure</xsl:if>
            </xsl:attribute>
            <xsl:value-of select="$assertsNokCount"/></td>
            <td><xsl:value-of select="$assertsOkCount"/></td>
            <td>
	            <xsl:attribute name="class">
	                <xsl:if test="$assertsSuccessRate = 1">status success</xsl:if>
	            </xsl:attribute>
                <xsl:call-template name="display-percent">
                    <xsl:with-param name="value" select="$assertsSuccessRate"/>
                </xsl:call-template>
            </td>
        </tr>
        </table>
        <p class="note">Note: <em class="status failure">failures</em> are anticipated and checked for with assertions while <em class="status error">errors</em> are unanticipated. </p>
        <p class="note">Note: <em class="status ignored">ignored</em> tests are executed but not counted on the success rate. </p>
        <p class="note">Note: <em class="status triedAgain">tried again</em> tests are executed but not counted on the success rate. </p>
    
    </xsl:template>

<xsl:template match="testsuite" mode="header">
    <tr>
        <th width="70%">Name</th>
        <th>Tests</th>
        <th>Errors</th>
        <th>Failures</th>
        <th>Ignored</th>
        <th>Tried Again</th>
        <th nowrap="nowrap">Time(s)</th>
    </tr>
</xsl:template>

<!-- class header -->
<xsl:template name="testsuite.test.header">
    <tr>
        <th width="70%">Name</th>
        <th>Tests</th>
        <th>Errors</th>
        <th>Failures</th>
        <th>Ignored</th>
        <th>Tried Again</th>
        <th nowrap="nowrap">Time(s)</th>
        <th nowrap="nowrap">Time Stamp</th>
        <th>Host</th>
    </tr>
</xsl:template>

<!-- method header -->
<xsl:template name="testcase.test.header">
    <tr>
        <th>Name</th>
        <th>Status</th>
        <th width="80%">Type</th>
        <th nowrap="nowrap">Time(s)</th>
    </tr>
</xsl:template>


<!-- class information -->
<xsl:template match="testsuite" mode="print.test">
    <tr>
        <!-- print testsuite information -->
        <td><a href="#{@name}"><code><xsl:value-of select="@name"/></code></a></td>
        <td><xsl:value-of select="@tests"/></td>
        <td>
         <xsl:attribute name="class">
                <xsl:if test="@errors &gt; 0">status error</xsl:if>
            </xsl:attribute>
        <xsl:value-of select="@errors"/></td>
        <td>
         <xsl:attribute name="class">
                <xsl:if test="@failures &gt; 0">status failure</xsl:if>
            </xsl:attribute>
        <xsl:value-of select="@failures"/></td>
        <td>
        <xsl:attribute name="class">
                <xsl:if test="@ignored &gt; 0">status ignored</xsl:if>
            </xsl:attribute>
        <xsl:value-of select="@ignored"/></td>
        <td>
        <xsl:attribute name="class">
                <xsl:if test="@triedAgain &gt; 0">status triedAgain</xsl:if>
            </xsl:attribute>
        <xsl:value-of select="@triedAgain"/></td>
        <td>
            <xsl:call-template name="display-time">
                <xsl:with-param name="value" select="@time"/>
            </xsl:call-template>
        </td>
        <td><xsl:apply-templates select="@timestamp"/></td>
        <td><xsl:apply-templates select="@hostname"/></td>
    </tr>
</xsl:template>

<xsl:template match="testcase" mode="print.test">
    <tr>
        <td><code><xsl:value-of select="@name"/></code></td>
        <xsl:choose>
        	<xsl:when test="(../@ignored!=0) and (failure)">
    	         <td class="status ignored">Ignored (Failure)</td>
                <td><xsl:apply-templates select="failure"/></td>
            </xsl:when>
        	<xsl:when test="(../@ignored!=0) and (error)">
    	         <td class="status ignored">Ignored (Error)</td>
                <td><xsl:apply-templates select="error"/></td>
            </xsl:when>
            <xsl:when test="(../@ignored!=0)">
    	         <td class="status ignored">Ignored (Success)</td>
                <td><xsl:apply-templates select="system-out"/></td>
            </xsl:when>
            <xsl:when test="(../@triedAgain!=0) and (failure)">
    	         <td class="status triedAgain">Tried Again Later (Failure)</td>
                <td><xsl:apply-templates select="failure"/></td>
            </xsl:when>
        	<xsl:when test="(../@triedAgain!=0) and (error)">
    	         <td class="status triedAgain">Tried Again Later (Error)</td>
                <td><xsl:apply-templates select="error"/></td>
            </xsl:when>
            <xsl:when test="(../@triedAgain!=0)">
    	         <td class="status triedAgain">Tried Again Later (Success)</td>
                <td><xsl:apply-templates select="system-out"/></td>
            </xsl:when>
            <xsl:when test="failure">
                <td class="status failure">Failure</td>
                <td><xsl:apply-templates select="failure"/></td>
            </xsl:when>
            <xsl:when test="error">
                <td class="status error">Error</td>
                <td><xsl:apply-templates select="error"/></td>
            </xsl:when>
            <xsl:otherwise>
                <td class="status success">Success</td>
                <td><xsl:apply-templates select="system-out"/></td>
            </xsl:otherwise>
        </xsl:choose>
        <td>
            <xsl:call-template name="display-time">
                <xsl:with-param name="value" select="@time"/>
            </xsl:call-template>
        </td>
    </tr>
</xsl:template>

<xsl:template match="failure">
    <xsl:call-template name="display-failures"/>
</xsl:template>

<xsl:template match="error">
    <xsl:call-template name="display-failures"/>
</xsl:template>

<xsl:template match="system-out">
    <xsl:call-template name="display-failures"/>
</xsl:template>

<!-- Style for the error and failure in the testcase template -->
<xsl:template name="display-failures">
    <xsl:choose>
        <xsl:when test="not(@message)">N/A</xsl:when>
        <xsl:otherwise>
            <xsl:value-of select="@message"/>
        </xsl:otherwise>
    </xsl:choose>
    <!-- display the stacktrace -->
    <pre><code><xsl:value-of select="." disable-output-escaping="yes"/></code></pre>
    <!-- the later is better but might be problematic for non-21" monitors... -->
    <!--pre><xsl:value-of select="."/></pre-->
</xsl:template>

<xsl:template name="display-time">
    <xsl:param name="value"/>
    <xsl:value-of select="format-number($value,'0.000')"/>
</xsl:template>

<xsl:template name="display-percent">
    <xsl:param name="value"/>
    <xsl:value-of select="format-number($value,'0.00%')"/>
</xsl:template>

</xsl:stylesheet>
