/*
 * Java
 *
 * Copyright 2019-2021 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.verisilicon.vivante;

/**
 * Provides access to vg_lite native APIs specific for the matrix operations.
 */
public class vg_lite_matrix implements AutoCloseable {

	/**
	 * Creates an identity matrix.
	 * <p>
	 * Important: On call of this constructor a dynamic allocation is done. To free this allocation the method close
	 * must be called.
	 * <p>
	 * If the instance is dereferenced without being closed, a memory leak will be generated
	 */
	public vg_lite_matrix() {
		throw new RuntimeException();
	}

	/**
	 * Creates a matrix from an existing handle.
	 * <p>
	 * Important: On call of this constructor dynamic allocation is NOT done. The close method of the created instance
	 * must be called only once.
	 *
	 * @param handle
	 *            handle on an existing matrix
	 */
	public vg_lite_matrix(int handle) {
		throw new RuntimeException();
	}

	@Override
	public void close() {
		throw new RuntimeException();
	}

	/**
	 * Gets the handle of a matrix.
	 * 
	 * @return the handle of the matrix.
	 */
	public int getHandle() {
		throw new RuntimeException();
	}

	/**
	 * Load the identity matrix.
	 * <p>
	 * See <code>vg_lite_identity</code> C function.
	 *
	 * @param matrix
	 *            matrix on which to execute operation
	 */
	public static void vg_lite_identity(vg_lite_matrix matrix) {
		throw new RuntimeException();
	}

	/**
	 * Translate the matrix to a new position.
	 * <p>
	 * See <code>vg_lite_translate</code> C function.
	 *
	 * @param x
	 *            X location of the transformation.
	 * @param y
	 *            Y location of the transformation.
	 * @param matrix
	 *            matrix on which to execute operation
	 */
	public static void vg_lite_translate(float x, float y, vg_lite_matrix matrix) {
		throw new RuntimeException();
	}

	/**
	 * Scale the matrix in both x and y directions.
	 * <p>
	 * See <code>vg_lite_scale</code> C function.
	 *
	 * @param scale_x
	 *            Horizontal scale.
	 * @param scale_y
	 *            Vertical scale.
	 * @param matrix
	 *            matrix on which to execute operation
	 */
	public static void vg_lite_scale(float scale_x, float scale_y, vg_lite_matrix matrix) {
		throw new RuntimeException();
	}

	/**
	 * Rotate the matrix.
	 * <p>
	 * See <code>vg_lite_rotate</code> C function.
	 *
	 * @param degrees
	 *            Number of degrees to rotate the matrix around. Positive numbers rotate counter clock wise.
	 * @param matrix
	 *            matrix on which to execute operation
	 */
	public static void vg_lite_rotate(float degrees, vg_lite_matrix matrix) {
		throw new RuntimeException();
	}

	/**
	 * Set perspective matrix.
	 * <p>
	 * See <code>vg_lite_perspective</code> C function.
	 *
	 * @param px
	 *            Indicate w0 of perspective transformation matrix.
	 * @param py
	 *            indicate w1 of perspective transformation matrix.
	 * @param matrix
	 *            matrix on which to execute operation
	 */
	public static void vg_lite_perspective(float px, float py, vg_lite_matrix matrix) {
		throw new RuntimeException();
	}
}
