/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package ej.giml;

import java.util.NoSuchElementException;

public interface GIMLFactoryProvider {

	/**
	 * Initializes the GIML factory provider. This method must be called before any operations.
	 *
	 * @param reader
	 *            the GIML reader used to read the GIML binary file to create GIML objects
	 *
	 */
	void initialize(GIMLReader reader);

	/**
	 * Creates a GIML object using the factory identified by the given id.
	 *
	 * @param factoryId
	 *            the factory identifier.
	 * @return the created GIML object.
	 *
	 * @throws NoSuchElementException
	 *             if no factory found with the given identifier to create the GIML object
	 */
	Object createGIMLObject(int factoryId) throws NoSuchElementException;

	/**
	 * Executes GIML specific code identified by the given id.
	 *
	 * @param codeId
	 *            the code identifier.
	 * @throws NoSuchElementException
	 *             if no factory found with the given identifier to create the GIML object
	 * @return the result of the execution on success
	 * @throws Throwable
	 *             any kind of exception or error can be thrown if execution fails
	 */
	Object executeGIMLCode(int codeId) throws Throwable;
}
