/*
 * Java
 *
 * Copyright 2016-2017 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package ej.giml;

import ej.nls.NLS;

/**
 * Reads GIML binary data.<p>
 * This class is not thread safe. Concurrent access to this object may lead to unknown behavior.
 * You may create several {@link GIMLReader} to access the same binary data.
 */
public interface GIMLReader {

	/**
	 * Initialize the reader. Must be called before any operations.
	 * @param nls the NLS instance to use to decode NLS strings.
	 */
	public void initialize(NLS nls);

	/**
	 * Reads a 32-bits signed integer and increments position in the data.
	 *
	 * @return the value read.
	 */
	public int readVarSInt();

	/**
	 * Reads a 32-bits unsigned integer and increments position in the data.
	 *
	 * @return the value read.
	 */
	public int readVarUInt();

	/**
	 * Reads a 64-bits signed integer and increments position in the data.
	 *
	 * @return the value read.
	 */
	public long readVarLong();

	/**
	 * Reads a boolean and increments position in the data.
	 *
	 * @return the value read.
	 */
	public boolean readBoolean();

	/**
	 * Reads a 32-bits float and increments position in the data.
	 *
	 * @return the value read.
	 */
	public float readFloat();

	/**
	 * Reads a 64-bits float and increments position in the data.
	 *
	 * @return the value read.
	 */
	public double readDouble();

	/**
	 * Reads a String and increments position in the data.
	 *
	 * @return the value read.
	 */
	public String readString();

	/**
	 * Sets the position pointer on the position identified by the given identifier.
	 * @param positionIdentifier the identifier of the position.
	 * @throws IllegalArgumentException if there is no position with the given identifier.
	 */
	public void seek(int positionIdentifier) throws IllegalArgumentException;

	/**
	 * Returns the GIML binary identifier.
	 * @return the GIML binary identifier.
	 */
	public long getBinaryId();

}
