/*
 * Java
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.vector.font;

/**
 * A multiple linear gradient to apply to fonts.
 * <p>
 * Note: {@link LinearGradient} objects are not tread safe and may be modified by calling {@link VectorFontPainter}
 * drawing methods. Thus, the same gradient should not be used in several threads.
 * <p>
 * Each created instance must be closed, calling {@link #close()} method, to free memory.
 */
public class LinearGradient implements AutoCloseable {

	/**
	 * Creates a new gradient instance.
	 *
	 * @param startX
	 *            the x coordinate of the start of the gradient
	 * @param startY
	 *            the y coordinate of the start of the gradient
	 * @param endX
	 *            the x coordinate of the end of the gradient
	 * @param endY
	 *            the y coordinate of the end of the gradient
	 * @param fractions
	 *            the colors distribution along the gradient from 0 (start) to 255 (end). Should have the same size as
	 *            colors.
	 * @param colors
	 *            the colors in ARGB format. Should as the same size as fractions.
	 */
	public LinearGradient(int startX, int startY, int endX, int endY, int[] fractions, int[] colors) {
		throw new RuntimeException();
	}

	@Override
	public void close() {
		throw new RuntimeException();
	}
}
