/*
 * Java
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.vector.font;

import ej.microui.display.GraphicsContext;
import ej.vector.Matrix;

/**
 * The <code>VectorFontPainter</code> class offers a set of static methods to draw in a {@link GraphicsContext}.
 * <p>
 * Only this class offers some drawing APIs in VectorFont library. All others classes ({@link VectorFont}) contain only
 * some contexts to specify the drawing to perform.
 * <p>
 * All drawings respect the colors set in {@link GraphicsContext} (background color is not used by all drawing methods).
 * <p>
 * The coordinate system is as follows:
 * <ul>
 * <li>origin is at the upper left corner of the destination.</li>
 * <li>X-axis is positive towards the right.</li>
 * <li>Y-axis is positive downwards.</li>
 * </ul>
 */
public class VectorFontPainter {

	private VectorFontPainter() {
		// static methods only
	}

	/**
	 * Draws the character on given graphics context and using graphics context color.
	 * <p>
	 * The character position, scale and orientation is determined by a transformation matrix.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the character.
	 * @param font
	 *            the {@link VectorFont} to use to render the character.
	 * @param height
	 *            the height of the text in pixels.
	 * @param character
	 *            the character to draw.
	 * @param matrix
	 *            the transformation matrix to apply.
	 * @param alpha
	 *            the opacity to apply to the character, from 0 (transparent) to 255 (fully opaque).
	 */
	public static void drawChar(GraphicsContext gc, VectorFont font, float height, char character, Matrix matrix,
			int alpha) {
		throw new RuntimeException();
	}

	/**
	 * Draws the character on given graphics context and using graphics context color.
	 * <p>
	 * The character position, scale and orientation is determined by a transformation matrix.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the character.
	 * @param font
	 *            the {@link VectorFont} to use to render the character.
	 * @param height
	 *            the height of the text in pixels.
	 * @param character
	 *            the character to draw.
	 * @param matrix
	 *            the transformation matrix to apply.
	 * @param gradient
	 *            the gradient to apply.
	 */
	public static void drawChar(GraphicsContext gc, VectorFont font, float height, char character, Matrix matrix,
			LinearGradient gradient) {
		throw new RuntimeException();
	}

	/**
	 * Draws the string on given graphics context and using graphics context color.
	 * <p>
	 * The text top-left anchor point is at position <code>(x,y)</code>. To fix the anchor point at baseline position,
	 * give <code>y- font.getBaselinePosition()</code> as parameter. The text anchor point is at position
	 * <code>(x,y)</code>.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param string
	 *            the string to draw.
	 * @param x
	 *            the x coordinate of the anchor top-left point.
	 * @param y
	 *            the y coordinate of the anchor top-left point.
	 */
	public static void drawString(GraphicsContext gc, VectorFont font, float height, String string, int x, int y) {
		throw new RuntimeException();
	}

	/**
	 * Draws the string on given graphics context and using graphics context color.
	 * <p>
	 * The text top-left anchor point is at position <code>(x,y)</code>. To fix the anchor point at baseline position,
	 * give <code>y- font.getBaselinePosition()</code> as parameter. The text anchor point is at position
	 * <code>(x,y)</code>.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param string
	 *            the string to draw.
	 * @param x
	 *            the x coordinate of the anchor top-left point.
	 * @param y
	 *            the y coordinate of the anchor top-left point.
	 * @param alpha
	 *            the opacity to apply to the string, from 0 (transparent) to 255 (fully opaque).
	 */
	public static void drawString(GraphicsContext gc, VectorFont font, float height, String string, int x, int y,
			int alpha) {
		throw new RuntimeException();
	}

	/**
	 * Draws the string on given graphics context and using graphics context color.
	 * <p>
	 * The text top-left anchor point is at position <code>(x,y)</code>. To fix the anchor point at baseline position,
	 * give <code>y- font.getBaselinePosition()</code> as parameter. The text anchor point is at position
	 * <code>(x,y)</code>.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param string
	 *            the string to draw.
	 * @param x
	 *            the x coordinate of the anchor top-left point.
	 * @param y
	 *            the y coordinate of the anchor top-left point.
	 * @param gradient
	 *            the gradient to apply.
	 */
	public static void drawString(GraphicsContext gc, VectorFont font, float height, String string, int x, int y,
			LinearGradient gradient) {
		throw new RuntimeException();
	}

	/**
	 * Draws the char array on given graphics context and using graphics context color.
	 * <p>
	 * The text top-left anchor point is at position <code>(x,y)</code>. To fix the anchor point at baseline position,
	 * give <code>y- font.getBaselinePosition()</code> as parameter. The text anchor point is at position
	 * <code>(x,y)</code>.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param chars
	 *            the char array to draw.
	 * @param offset
	 *            the offset of the first character to draw.
	 * @param length
	 *            the number of characters to draw from the array.
	 * @param x
	 *            the x coordinate of the anchor top-left point.
	 * @param y
	 *            the y coordinate of the anchor top-left point.
	 */
	public static void drawChars(GraphicsContext gc, VectorFont font, float height, char[] chars, int offset,
			int length, int x, int y) {
		throw new RuntimeException();
	}

	/**
	 * Draws the char array on given graphics context and using graphics context color.
	 * <p>
	 * The text top-left anchor point is at position <code>(x,y)</code>. To fix the anchor point at baseline position,
	 * give <code>y- font.getBaselinePosition()</code> as parameter. The text anchor point is at position
	 * <code>(x,y)</code>.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param chars
	 *            the char array to draw.
	 * @param offset
	 *            the offset of the first character to draw.
	 * @param length
	 *            the number of characters to draw from the array.
	 * @param x
	 *            the x coordinate of the anchor top-left point.
	 * @param y
	 *            the y coordinate of the anchor top-left point.
	 * @param alpha
	 *            the opacity to apply to the string, from 0 (transparent) to 255 (fully opaque).
	 */
	public static void drawChars(GraphicsContext gc, VectorFont font, float height, char[] chars, int offset,
			int length, int x, int y, int alpha) {
		throw new RuntimeException();
	}

	/**
	 * Draws the char array on given graphics context and using graphics context color.
	 * <p>
	 * The text top-left anchor point is at position <code>(x,y)</code>. To fix the anchor point at baseline position,
	 * give <code>y- font.getBaselinePosition()</code> as parameter. The text anchor point is at position
	 * <code>(x,y)</code>.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param chars
	 *            the char array to draw.
	 * @param offset
	 *            the offset of the first character to draw.
	 * @param length
	 *            the number of characters to draw from the array.
	 * @param x
	 *            the x coordinate of the anchor top-left point.
	 * @param y
	 *            the y coordinate of the anchor top-left point.
	 * @param gradient
	 *            the gradient to apply.
	 */
	public static void drawChars(GraphicsContext gc, VectorFont font, float height, char[] chars, int offset,
			int length, int x, int y, LinearGradient gradient) {
		throw new RuntimeException();
	}

	/**
	 * Draws the string around a circle on given graphics context and using graphics context color.
	 * <p>
	 * The string will be drawn inside the circle (the maximum height of the characters is on the circle outline)
	 * starting at the given angle.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param string
	 *            the string to draw.
	 * @param x
	 *            the x coordinate of the circle top-left point.
	 * @param y
	 *            the y coordinate of the circle top-left point.
	 * @param diameter
	 *            the diameter of the circle.
	 * @param angle
	 *            the angle in degrees at which the string starts.
	 */
	public static void drawStringArc(GraphicsContext gc, VectorFont font, float height, String string, int x, int y,
			int diameter, float angle) {
		throw new RuntimeException();
	}

	/**
	 * Draws the string around a circle on given graphics context and using graphics context color.
	 * <p>
	 * The string will be drawn inside the circle (the maximum height of the characters is on the circle outline)
	 * starting at the given angle.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param string
	 *            the string to draw.
	 * @param x
	 *            the x coordinate of the circle top-left point.
	 * @param y
	 *            the y coordinate of the circle top-left point.
	 * @param diameter
	 *            the diameter of the circle.
	 * @param angle
	 *            the angle in degrees at which the string starts.
	 * @param alpha
	 *            the opacity to apply to the string, from 0 (transparent) to 255 (fully opaque).
	 */
	public static void drawStringArc(GraphicsContext gc, VectorFont font, float height, String string, int x, int y,
			int diameter, float angle, int alpha) {
		throw new RuntimeException();
	}

	/**
	 * Draws the string around a circle on given graphics context and using graphics context color.
	 * <p>
	 * The string will be drawn inside the circle (the maximum height of the characters is on the circle outline)
	 * starting at the given angle.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param string
	 *            the string to draw.
	 * @param x
	 *            the x coordinate of the circle top-left point.
	 * @param y
	 *            the y coordinate of the circle top-left point.
	 * @param diameter
	 *            the diameter of the circle.
	 * @param angle
	 *            the angle in degrees at which the string starts.
	 * @param flip
	 *            tells whether the text should be written using the circle outline as baseline. This allows to draw the
	 *            text at the bottom of the circle right side up when parameter is true.
	 */
	public static void drawStringArc(GraphicsContext gc, VectorFont font, float height, String string, int x, int y,
			int diameter, float angle, boolean flip) {
		throw new RuntimeException();
	}

	/**
	 * Draws the string around a circle on given graphics context and using graphics context color.
	 * <p>
	 * The string will be drawn inside the circle (the maximum height of the characters is on the circle outline)
	 * starting at the given angle.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param string
	 *            the string to draw.
	 * @param x
	 *            the x coordinate of the circle top-left point.
	 * @param y
	 *            the y coordinate of the circle top-left point.
	 * @param diameter
	 *            the diameter of the circle.
	 * @param angle
	 *            the angle in degrees at which the string starts.
	 * @param alpha
	 *            the opacity to apply to the string, from 0 (transparent) to 255 (fully opaque).
	 * @param flip
	 *            tells whether the text should be written using the circle outline as baseline. This allows to draw the
	 *            text at the bottom of the circle right side up when parameter is true.
	 */
	public static void drawStringArc(GraphicsContext gc, VectorFont font, float height, String string, int x, int y,
			int diameter, float angle, int alpha, boolean flip) {
		throw new RuntimeException();
	}

	/**
	 * Draws the string around a circle on given graphics context and using graphics context color.
	 * <p>
	 * The string will be drawn inside the circle (the maximum height of the characters is on the circle outline)
	 * starting at the given angle.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param string
	 *            the string to draw.
	 * @param x
	 *            the x coordinate of the circle top-left point.
	 * @param y
	 *            the y coordinate of the circle top-left point.
	 * @param diameter
	 *            the diameter of the circle.
	 * @param angle
	 *            the angle in degrees at which the string starts.
	 * @param gradient
	 *            the gradient to apply.
	 */
	public static void drawStringArc(GraphicsContext gc, VectorFont font, float height, String string, int x, int y,
			int diameter, float angle, LinearGradient gradient) {
		throw new RuntimeException();
	}

	/**
	 * Draws the string around a circle on given graphics context and using graphics context color.
	 * <p>
	 * The string will be drawn inside the circle (the maximum height of the characters is on the circle outline)
	 * starting at the given angle.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param string
	 *            the string to draw.
	 * @param x
	 *            the x coordinate of the circle top-left point.
	 * @param y
	 *            the y coordinate of the circle top-left point.
	 * @param diameter
	 *            the diameter of the circle.
	 * @param angle
	 *            the angle in degrees at which the string starts.
	 * @param gradient
	 *            the opacity of the text to draw (0xFF is fully opaque).
	 * @param flip
	 *            tells whether the text should be written using the circle outline as baseline. This allows to draw the
	 *            text at the bottom of the circle right side up when parameter is true.
	 */
	public static void drawStringArc(GraphicsContext gc, VectorFont font, float height, String string, int x, int y,
			int diameter, float angle, LinearGradient gradient, boolean flip) {
		throw new RuntimeException();
	}

	/**
	 * Draws the char array around a circle on given graphics context and using graphics context color.
	 * <p>
	 * The string will be drawn inside the circle (the maximum height of the characters is on the circle outline)
	 * starting at the given angle.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param chars
	 *            the char array to draw.
	 * @param offset
	 *            the offset of the first character to draw.
	 * @param length
	 *            the number of characters to draw from the array.
	 * @param x
	 *            the x coordinate of the circle top-left point.
	 * @param y
	 *            the y coordinate of the circle top-left point.
	 * @param diameter
	 *            the diameter of the circle.
	 * @param angle
	 *            the angle in degrees at which the string starts.
	 */
	public static void drawCharsArc(GraphicsContext gc, VectorFont font, float height, char[] chars, int offset,
			int length, int x, int y, int diameter, float angle) {
		throw new RuntimeException();
	}

	/**
	 * Draws the char array around a circle on given graphics context and using graphics context color.
	 * <p>
	 * The string will be drawn inside the circle (the maximum height of the characters is on the circle outline)
	 * starting at the given angle.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param chars
	 *            the char array to draw.
	 * @param offset
	 *            the offset of the first character to draw.
	 * @param length
	 *            the number of characters to draw from the array.
	 * @param x
	 *            the x coordinate of the circle top-left point.
	 * @param y
	 *            the y coordinate of the circle top-left point.
	 * @param diameter
	 *            the diameter of the circle.
	 * @param angle
	 *            the angle in degrees at which the string starts.
	 * @param alpha
	 *            the opacity to apply to the string, from 0 (transparent) to 255 (fully opaque).
	 */
	public static void drawCharsArc(GraphicsContext gc, VectorFont font, float height, char[] chars, int offset,
			int length, int x, int y, int diameter, float angle, int alpha) {
		throw new RuntimeException();
	}

	/**
	 * Draws the char array around a circle on given graphics context and using graphics context color.
	 * <p>
	 * The string will be drawn inside the circle (the maximum height of the characters is on the circle outline)
	 * starting at the given angle.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param chars
	 *            the char array to draw.
	 * @param offset
	 *            the offset of the first character to draw.
	 * @param length
	 *            the number of characters to draw from the array.
	 * @param x
	 *            the x coordinate of the circle top-left point.
	 * @param y
	 *            the y coordinate of the circle top-left point.
	 * @param diameter
	 *            the diameter of the circle.
	 * @param angle
	 *            the angle in degrees at which the string starts.
	 * @param flip
	 *            tells whether the text should be written using the circle outline as baseline. This allows to draw the
	 *            text at the bottom of the circle right side up when parameter is true.
	 */
	public static void drawCharsArc(GraphicsContext gc, VectorFont font, float height, char[] chars, int offset,
			int length, int x, int y, int diameter, float angle, boolean flip) {
		throw new RuntimeException();
	}

	/**
	 * Draws the char array around a circle on given graphics context and using graphics context color.
	 * <p>
	 * The string will be drawn inside the circle (the maximum height of the characters is on the circle outline)
	 * starting at the given angle.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param chars
	 *            the char array to draw.
	 * @param offset
	 *            the offset of the first character to draw.
	 * @param length
	 *            the number of characters to draw from the array.
	 * @param x
	 *            the x coordinate of the circle top-left point.
	 * @param y
	 *            the y coordinate of the circle top-left point.
	 * @param diameter
	 *            the diameter of the circle.
	 * @param angle
	 *            the angle in degrees at which the string starts.
	 * @param alpha
	 *            the opacity to apply to the string, from 0 (transparent) to 255 (fully opaque).
	 * @param flip
	 *            tells whether the text should be written using the circle outline as baseline. This allows to draw the
	 *            text at the bottom of the circle right side up when parameter is true.
	 */
	public static void drawCharsArc(GraphicsContext gc, VectorFont font, float height, char[] chars, int offset,
			int length, int x, int y, int diameter, float angle, int alpha, boolean flip) {
		throw new RuntimeException();
	}

	/**
	 * Draws the char array around a circle on given graphics context and using graphics context color.
	 * <p>
	 * The string will be drawn inside the circle (the maximum height of the characters is on the circle outline)
	 * starting at the given angle.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param chars
	 *            the char array to draw.
	 * @param offset
	 *            the offset of the first character to draw.
	 * @param length
	 *            the number of characters to draw from the array.
	 * @param x
	 *            the x coordinate of the circle top-left point.
	 * @param y
	 *            the y coordinate of the circle top-left point.
	 * @param diameter
	 *            the diameter of the circle.
	 * @param angle
	 *            the angle in degrees at which the string starts.
	 * @param gradient
	 *            the gradient to apply.
	 */
	public static void drawCharsArc(GraphicsContext gc, VectorFont font, float height, char[] chars, int offset,
			int length, int x, int y, int diameter, float angle, LinearGradient gradient) {
		throw new RuntimeException();
	}

	/**
	 * Draws the char array around a circle on given graphics context and using graphics context color.
	 * <p>
	 * The string will be drawn inside the circle (the maximum height of the characters is on the circle outline)
	 * starting at the given angle.
	 *
	 * @param gc
	 *            the {@link GraphicsContext} where to render the string.
	 * @param font
	 *            the {@link VectorFont} to use to render the string.
	 * @param height
	 *            the height of the text in pixels.
	 * @param chars
	 *            the char array to draw.
	 * @param offset
	 *            the offset of the first character to draw.
	 * @param length
	 *            the number of characters to draw from the array.
	 * @param x
	 *            the x coordinate of the circle top-left point.
	 * @param y
	 *            the y coordinate of the circle top-left point.
	 * @param diameter
	 *            the diameter of the circle.
	 * @param angle
	 *            the angle in degrees at which the string starts.
	 * @param gradient
	 *            the gradient to apply.
	 * @param flip
	 *            tells whether the text should be written using the circle outline as baseline. This allows to draw the
	 *            text at the bottom of the circle right side up when parameter is true.
	 */
	public static void drawCharsArc(GraphicsContext gc, VectorFont font, float height, char[] chars, int offset,
			int length, int x, int y, int diameter, float angle, LinearGradient gradient, boolean flip) {
		throw new RuntimeException();
	}
}
